﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WSMeter.Link
{
    abstract class Link
    {
        public event EventHandler Connected = (s, e) => { };
        public event EventHandler Disconnected = (s, e) => { };
        public event SerialDataEventHandler StringReceived = (s, e) => { };

        protected bool mConnected = false;
        bool mManual = false;

        public bool Manual
        {
            get { return mManual; }
            set
            {
                if (value != mManual)
                {
                    mManual = value;
                    // ...
                }
            }
        }

        private string mTempResult = "";

        /// <summary>
        /// Установить связь
        /// </summary>
        public virtual void Connect()
        {

        }

        /// <summary>
        /// Разорвать связь
        /// </summary>
        public virtual void Disconnect()
        {

        }

        /// <summary>
        /// Отправка строки
        /// </summary>
        /// <param name="Value"></param>
        public virtual void Send(string Value)
        {

        }

        /// <summary>
        /// Установлена ли связь с устройством?
        /// </summary>
        public virtual bool IsConnected
        {
            get { return mConnected; }
            internal set { mConnected = value; }
        }

        /// <summary>
        /// Вызвать событие, что связь установлена
        /// </summary>
        protected void FireConnected()
        {
            Connected(this, new EventArgs());
        }

        /// <summary>
        /// Вызвать событие, что связь разорвана
        /// </summary>
        protected void FireDisconnected()
        {
            Disconnected(this, new EventArgs());
        }

        /// <summary>
        /// Вызвать событие, что принята строка
        /// </summary>
        protected void FireStringReceived(string Text)
        {
            StringReceived(this, new SerialDataEventArgs(Text));
        }

        /// <summary>
        /// Обработать очередной символ
        /// </summary>
        /// <param name="C"></param>
        protected void ProcessChar(char C)
        {
            if(C == '\n')
            {
                FireStringReceived(mTempResult);
                mTempResult = "";
            }
            else
                mTempResult += C;
        }
    }

    class SerialDataEventArgs : EventArgs
    {
        public string Text = "";

        public SerialDataEventArgs(string Text)
        {
            this.Text = string.Copy(Text);
        }
    }

    delegate void SerialDataEventHandler(object sender, SerialDataEventArgs e);
}
