﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

using Network;
using System.Net;
using Extension.IP;
using System.Diagnostics;

namespace WSMeter.Link
{
    class NetworkLink : Link
    {
        public event IPEventHandler onDeviceFound = (o, e) => { };

        DateTime LastConnected;
        UDP mU;
        //UDP mUSend;
        IPAddress mAddress = null;

        List<IPAddress> mDevices = new List<IPAddress>();

        public NetworkLink()
        {
            //mUSend = new UDPSocket(NetworkSelector.getDefaultInterface(), 12345, 5554);
            mU = new UDPSocket(NetworkSelector.getDefaultInterface(), 5554);
            mU.FilterLoopback = true;
            mU.onPacketReceive += mU_onPacketReceive;
        }

        public NetworkLink(int Local, int Remote)
        {
            //mUSend = new UDPSocket(NetworkSelector.getDefaultInterface(), 12345, 5554);
            mU = new UDPSocket(NetworkSelector.getDefaultInterface(), Local, Remote);
            mU.FilterLoopback = true;
            mU.onPacketReceive += mU_onPacketReceive;
        }

        public NetworkLink(string IP, int Local, int Remote)
        {
            //mUSend = new UDPSocket(NetworkSelector.getInterface(IP), 12345, 5554);
            mU = new UDPSocket(NetworkSelector.getInterface(IP), Local, Remote);
            mU.FilterLoopback = true;
            mU.onPacketReceive += mU_onPacketReceive;
        }

        public IPAddress Address
        {
            get { return mAddress; }
            set { mAddress = value; }
        }

        public void Find()
        {
            if (mAddress != null)
                mU.Send(mAddress.getBroadcastAddress(), new Packet.WSPacket("FIND\n").getPacket());
        }

        public void Ping()
        {
            if (Manual) return;

            Send("PING\n");
            if(IsConnected)
            {
                if((DateTime.Now - LastConnected).TotalSeconds > 4)
                {
                    IsConnected = false;
                    FireDisconnected();
                }
            }
        }

        /// <summary>
        /// Отправка строки
        /// </summary>
        /// <param name="Value"></param>
        public override void Send(string Value)
        {
            if(mAddress != null)
                mU.Send(mAddress, new Packet.WSPacket(Value).getPacket());
        }

        /// <summary>
        /// Отправка строки всем
        /// </summary>
        /// <param name="Value"></param>
        public void SendBroadcast(string Value)
        {
            if (mAddress != null)
                mU.Send(new Packet.WSPacket(Value).getPacket());
        }

        /// <summary>
        /// Отправка строки всем
        /// </summary>
        /// <param name="Value"></param>
        public void SendBroadcast(NetworkInterface I, string Value)
        {
            if (mAddress != null)
                mU.Send(I.Broadcast, new Packet.WSPacket(Value).getPacket());
        }

        void mU_onPacketReceive(object sender, UDPPacketEventArgs E)
        {
            var Packet = new Packet.WSPacket(E.Packet);
            var Response = (from T in Packet.Text.Split(new char[] { '\n' }) select T.Trim()).ToArray();

            if (E.Sender.Equals(mAddress))
            {
                if (!IsConnected)
                {
                    IsConnected = true;
                    FireConnected();
                }
                LastConnected = DateTime.Now;

                foreach (var T in Response)
                {
                    if (T.Length == 0) continue;

                    switch (T)
                    {
                        case "RESPONSE":
                            onDeviceFound(this, new IPEventArgs(E.Sender));
                            if (!mDevices.Contains(E.Sender))
                            {
                                Debug.WriteLine("Found: " + E.Sender.ToString());
                                mDevices.Add(E.Sender);
                            }
                            break;
                        default:
                            FireStringReceived(T);
                            break;
                    }
                }
            }
            else
            {
                foreach (var T in Response)
                {
                    if (T.Length == 0) continue;

                    switch (T)
                    {
                        case "RESPONSE":
                            onDeviceFound(this, new IPEventArgs(E.Sender));
                            if (!mDevices.Contains(E.Sender))
                            {
                                Debug.WriteLine("Found: " + E.Sender.ToString());
                                mDevices.Add(E.Sender);
                            }
                            break;
                    }
                }
            }
        }
    }

    class IPEventArgs : EventArgs
    {
        public IPAddress IP;

        public IPEventArgs(IPAddress Address)
        {
            IP = Address;
        }
    }

    delegate void IPEventHandler(object sender, IPEventArgs e);
}
