﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO.Ports;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace WSMeter.Link
{
    class SerialLink : Link
    {
        private SerialPort SPort;

        public SerialLink()
        {
            SPort = new SerialPort();
            SPort.PortName = "COM0";
            SPort.BaudRate = 250000;
            SPort.Parity = Parity.None;
            SPort.DataBits = 8;
            SPort.StopBits = StopBits.Two;
            SPort.Handshake = Handshake.None;
        }

        static protected string[] AvailablePorts()
        {
            return SerialPort.GetPortNames();
        }

        void SPort_DataReceived(object sender, SerialDataReceivedEventArgs e)
        {
            SerialPort sp = (SerialPort)sender;

            int Size = sp.BytesToRead;
            byte[] Temp = new byte[Size];

            if (Size == 0) return;
            sp.Read(Temp, 0, Size);

            foreach (byte B in Temp) ProcessChar(Convert.ToChar(B));
        }

        public string Port
        {
            get { return SPort.PortName; }
            set { SetPort(value); }
        }

        public void SetPort(string Port)
        {
            SPort.PortName = Port;
        }

        /// <summary>
        /// Установить связь
        /// </summary>
        public override void Connect()
        {
            try
            {
                SPort.DataReceived += SPort_DataReceived;
                SPort.Open();
                SPort.DiscardInBuffer();
                SPort.Encoding = Encoding.GetEncoding(1251);
                IsConnected = true;
                FireConnected();
            }
            catch (Exception E)
            {

            }
        }

        /// <summary>
        /// Разорвать связь
        /// </summary>
        public override void Disconnect()
        {
            if (SPort.IsOpen)
            {
                try
                {
                    SPort.DataReceived -= SPort_DataReceived;
                    SPort.Close();
                }
                finally
                {
                    IsConnected = false;
                    FireDisconnected();
                }
            }
        }

        /// <summary>
        /// Отправка строки
        /// </summary>
        /// <param name="Value"></param>
        public override void Send(string Value)
        {
            if (IsConnected) SPort.WriteLine(Value);
        }
    }
}
