﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;

namespace Network
{
    public class NetworkInterface
    {
        /// <summary>
        /// IP-адрес
        /// </summary>
        public IPAddress Address;

        /// <summary>
        /// Маска сети
        /// </summary>
        public IPAddress Mask;

        /// <summary>
        /// Название
        /// </summary>
        public string Name;

        public IPAddress Broadcast
        {
            get
            {
                if(Address.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                {
                    var mBroadcastAddress = Address.GetAddressBytes();
                    var mMask = (Mask != null) ? Mask.GetAddressBytes() : new byte[] { 0, 0, 0, 0 };

                    for (int i = 0; i < 4; i++)
                        mBroadcastAddress[i] |= Convert.ToByte(~mMask[i] & 0xFF);

                    return new IPAddress(mBroadcastAddress);
                }
                return null;
            }
        }

        public NetworkInterface(IPAddress A, string N)
        {
            Address = A;
            Name = N;
        }

        public NetworkInterface(IPAddress A, IPAddress M, string N)
        {
            Address = A;
            Mask = M;
            Name = N;
        }

        public override string ToString()
        {
            if (Address == null)
                return "Non-exists interface";

            if (Address.Equals(IPAddress.Any))
                return "Automatic";
            else
                return Address.ToString();
        }
    }
}
