﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;

using Extension.IP;

namespace Network
{
    public static class NetworkSelector
    {
        /// <summary>
        /// Список сетевых интерфейсов
        /// </summary>
        /// <returns></returns>
        public static List<NetworkInterface> getInterfaces(bool AddAuto)
        {
            List<NetworkInterface> Interfaces = new List<NetworkInterface>();

            if(AddAuto) Interfaces.Add(new NetworkInterface(IPAddress.Any, IPAddress.Any, "Автоматически"));

            foreach (var f in System.Net.NetworkInformation.NetworkInterface.GetAllNetworkInterfaces())
            {
                var ipInterface = f.GetIPProperties();

                var Local = ipInterface.UnicastAddresses;

                foreach(var A in Local)
                {
                    if (A.Address.AddressFamily == System.Net.Sockets.AddressFamily.InterNetwork)
                        Interfaces.Add(new NetworkInterface(A.Address, A.IPv4Mask, A.ToString()));
                }
            }
            return Interfaces;
        }

        /// <summary>
        /// Список сетевых интерфейсов
        /// </summary>
        /// <returns></returns>
        public static List<NetworkInterface> getInterfaces()
        {
            return getInterfaces(true);
        }
        
        /// <summary>
        /// Получить мой адрес из этой же сети
        /// </summary>
        /// <param name="Remote"></param>
        public static IPAddress getMyIPFromNet(IPAddress Remote)
        {
            var I = getInterfaces();
            int Max = 0;
            IPAddress Result = IPAddress.Any;

            foreach(var If in I)
            {
                int V = Remote.GetSimilarLength(If.Address);

                if(V > Max)
                {
                    Max = V;
                    Result = If.Address;
                }
            }

            return Result;
        }

        /// <summary>
        /// Дефолтный интерфейс
        /// </summary>
        /// <returns></returns>
        public static NetworkInterface getDefaultInterface()
        {
            return new NetworkInterface(IPAddress.Any, "Автоматически");
        }

        /// <summary>
        /// Интерфейс по IP
        /// </summary>
        /// <returns></returns>
        public static NetworkInterface getInterface(string IP)
        {
            if(HasSuchInterface(IP))
            {
                var Ifs = getInterfaces();
                foreach (var I in Ifs)
                {
                    if (I.Address.ToString().CompareTo(IP) == 0) return I;
                }
            }

            return new NetworkInterface(IPAddress.Parse(IP), IP);
        }

        /// <summary>
        /// Есть ли подобный интерфейс
        /// </summary>
        /// <param name="IP"></param>
        /// <returns></returns>
        public static bool HasSuchInterface(string IP)
        {
            var Ifs = getInterfaces();

            foreach (var I in Ifs)
            {
                if (I.Address.ToString().CompareTo(IP) == 0) return true;
            }

            return false;
        }
    }
}
