﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Net;

namespace Network
{
    public class UDPPacketEventArgs
    {
        public IPAddress Sender;

        public byte[] Packet;

        public UDPPacketEventArgs(IPAddress A, byte[] D)
        {
            Sender = A;
            Packet = D;
        }
    }

    public delegate void UDPPacketEventHandler(object sender, UDPPacketEventArgs E);

    public abstract class UDP
    {
        private NetworkInterface mI;
        protected int mPort;

        public event UDPPacketEventHandler onPacketReceive;
        private bool mFilterLocal = true;

        public UDP(NetworkInterface I, int Port)
        {
            mI = I;
            mPort = Port;
        }

        /// <summary>
        /// Получить алрес сокета
        /// </summary>
        /// <returns></returns>
        public IPAddress getIP()
        {
            return mI.Address;
        }

        /// <summary>
        /// Переподключить сокет
        /// </summary>
        /// <param name="I"></param>
        public void Rebind(NetworkInterface I)
        {
            if (mI.Address.Equals(I.Address)) return;

            RebindInternal(I, mPort);
            mI = I;
        }

        /// <summary>
        /// Не принимать пакеты с адреса этого сокета
        /// </summary>
        public bool FilterLoopback
        {
            get { return mFilterLocal; }
            set { mFilterLocal = value; }
        }

        /// <summary>
        /// Переподключить
        /// </summary>
        /// <param name="I"></param>
        protected abstract void RebindInternal(NetworkInterface I, int Port);

        /// <summary>
        /// Адресный пакет
        /// </summary>
        /// <param name="EP"></param>
        /// <param name="Packet"></param>
        public abstract void Send(IPEndPoint EP, byte[] Packet);

        /// <summary>
        /// Адресный пакет
        /// </summary>
        /// <param name="IP"></param>
        /// <param name="Packet"></param>
        public abstract void Send(IPAddress IP, byte[] Packet);

        /// <summary>
        /// Адресный пакет
        /// </summary>
        /// <param name="IP"></param>
        /// <param name="Packet"></param>
        /// <param name="Port"></param>
        public abstract void Send(IPAddress IP, byte[] Packet, int Port);

        /// <summary>
        /// Широковещательный пакет
        /// </summary>
        /// <param name="Packet"></param>
        public abstract void Send(byte[] Packet);

        /// <summary>
        /// Вызвать обарботчик принятого покета
        /// </summary>
        /// <param name="A"></param>
        /// <param name="Data"></param>
        protected void fireOnPacket(IPAddress A, byte[] Data)
        {
            if (mFilterLocal && mI.Address.Equals(A)) return;

            if (onPacketReceive != null) onPacketReceive(this, new UDPPacketEventArgs(A, Data));
        }
    }
}
