﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using WSMeter.Link;
using Extension.Parameter;
using System.Net;
using WSMeter.Output;
using System.Globalization;
using System.IO;

namespace WSMeter
{
    public partial class fMain : Form
    {
        //SerialLink L = new SerialLink();
        Config C = Config.Load();
        NetworkLink NL;
        ProgramMode Mode = ProgramMode.Unknown;
        //bool ManualEdit = false;
        TextBox[] Inputs;

        public fMain()
        {
            InitializeComponent();
            NL = new NetworkLink(C.LocalPort, C.RemotePort);
        }

        public fMain(string IP)
        {
            InitializeComponent();

            NL = new NetworkLink(IP, C.LocalPort, C.RemotePort);
        }

        private void fMain_Load(object sender, EventArgs e)
        {
          //  L.Connected += L_Connected;
          //  L.Disconnected += L_Disconnected;
            //L.StringReceived += L_StringReceived;
           // L.SetPort("COM4");
            //  L.Connect();
            Inputs = new TextBox[] { tLength, tWidth, tHeight, tVolume, tWeight, tVolWeight, tUser1, tUser2, tUser3, tBar };

            lUser1.Text = C.User1 + ":";
            lUser2.Text = C.User2 + ":";
            lUser3.Text = C.User3 + ":";

            NL.Address = IPAddress.Parse(C.DeviceIP);
            NL.StringReceived += L_StringReceived;
            NL.Connected += L_Connected;
            NL.Disconnected += L_Disconnected;

            EnableControl(false);
            ChangeMode(ProgramMode.NotConnected);
            tTimer.Start();
        }

        private void fMain_FormClosing(object sender, FormClosingEventArgs e)
        {
            //L.Disconnect();
            NL.Disconnect();

            tTimer.Stop();
        }

        void L_StringReceived(object sender, SerialDataEventArgs e)
        {
            if(e.Text.CompareTo("OK") != 0) 
                Debug.WriteLine(String.Format("Readed: {0:s}", e.Text));

            if (NL.Manual) return;

            BeginInvoke((MethodInvoker)delegate()
            {
                string ParamName = e.Text.getParameterName();
                switch (ParamName)
                {
                    case "OK": break;
                    case "STARTED": 
                        ChangeMode(ProgramMode.MeasureWait); 
                        break;
                    case "NOTSTARTED":
                        ChangeMode(ProgramMode.Ready);
                        MessageBox.Show(e.Text.getStringValue("Измерение не началось."), "Ошибка");
                        break;
                    case "ERROR":
                        if (Mode == ProgramMode.Calibrate)
                        {
                            // Не удалось
                            ChangeMode(ProgramMode.Ready);
                        }
                        break;
                    case "ZWSET": ChangeMode(ProgramMode.Ready); break;
                    case "LENGTH":  tLength.Text = e.Text.getStringValue("0"); break;
                    case "WIDTH":   tWidth.Text  = e.Text.getStringValue("0"); break;
                    case "HEIGHT":  tHeight.Text = e.Text.getStringValue("0"); break;
                    case "WEIGHT":  tWeight.Text = e.Text.getStringValue("0"); break;
                    case "VOLUME":  tVolume.Text = e.Text.getStringValue("0"); break;
                    case "VOLWGHT": tVolWeight.Text = e.Text.getStringValue("0"); break;
                    case "BAR":     tBar.Text = e.Text.getStringValue(""); break;
                    case "MANUAL": 
                        // ManualEdit = true; 
                        break;
                    case "COMPLETED": 
                        // В зависимости от показаний выбираем режим
                        if (!C.AutoSave)
                        {
                       //     ManualEdit = false;
                            ChangeMode(ProgramMode.Measure);
                        }
                        else
                        {
                            SaveToCSV();
                            ChangeMode(ProgramMode.Ready);
                        }
                        break;
                }
            });
        }

        void L_Disconnected(object sender, EventArgs e)
        {
            BeginInvoke((MethodInvoker)delegate() { 
                EnableControl(false);
                ChangeMode(ProgramMode.NotConnected);
            });
        }

        void L_Connected(object sender, EventArgs e)
        {
            
            BeginInvoke((MethodInvoker)delegate() {
                EnableControl(true); 
                ChangeMode(ProgramMode.Ready);
            });
        }

        private void EnableControl(bool Value)
        {
            Text = Value ? "WSMeter - Подключено" : "WSMeter";

            bSettings.Enabled = true;
         //   bOK.Enabled = Value;
        }

        private void bOK_Click(object sender, EventArgs e)
        {
            NL.Send("ADC0=344\nADC1=555\nADC2=666\nADS0=330440\nADS1=330540\nADS2=330640\nADS3=330740\n"); // ok
          //  NL.Send("START"); // ok
        }

        private void bError_Click(object sender, EventArgs e)
        {
            NL.SendBroadcast("FIND\n"); // ok
        }

        private void bSettings_Click(object sender, EventArgs e)
        {
            var F = new fSettings(NL, C);
            if(F.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                IPAddress A = IPAddress.Parse(C.DeviceIP);

                if(!NL.Address.Equals(A))
                    NL.Address = A;

                lUser1.Text = C.User1 + ":";
                lUser2.Text = C.User2 + ":";
                lUser3.Text = C.User3 + ":";
            }
        }

        private void tTimer_Tick(object sender, EventArgs e)
        {
            NL.Ping();
        }

        private double GetValue(string Value)
        {
            try
            {
                return String.IsNullOrEmpty(Value) ? 0 : double.Parse(Value, CultureInfo.InvariantCulture);
            }
            catch
            {
                return 0;
            }
        }
        
        private void SaveToCSV()
        {
            // 12.03.2015_13.45.csv
            // https://msdn.microsoft.com/ru-ru/library/8kb3ddd4(v=vs.110).aspx - datetime format
            string FileName = C.CSVDir + DateTime.Now.ToString("dd.MM.yyyy_HH.mm") + ".csv";
            if (File.Exists(FileName)) FileName = C.CSVDir + DateTime.Now.ToString("dd.MM.yyyy_HH.mm.ss") + ".csv";

            var CSV = new MeasureCSV();
            CSV.Length = GetValue(tLength.Text);
            CSV.Width = GetValue(tWidth.Text);
            CSV.Height = GetValue(tHeight.Text);
            CSV.Volume = GetValue(tVolume.Text);
            CSV.Weight = GetValue(tWeight.Text);
            CSV.VolumeWeight = GetValue(tVolWeight.Text);
            CSV.Bar = tBar.Text;
            CSV.UserValue1 = tUser1.Text;
            CSV.UserValue2 = tUser2.Text;
            CSV.UserValue3 = tUser3.Text;

            if (C.UseCSVFile)
            {
                var Dir = Path.GetDirectoryName(C.CSVFile);
                if (!Directory.Exists(Dir))
                {
                    try
                    {
                        Directory.CreateDirectory(Dir);
                    }
                    catch (Exception E)
                    {
                        MessageBox.Show($"Не удаётся сохранить результат. Ошибка при создании папки {Dir}: {E.Message}");
                        return;
                    }
                }

                CSV.Append(C.CSVFile);
            }
            else
            {
                if (!Directory.Exists(C.CSVDir)) Directory.CreateDirectory(C.CSVDir);
                
                CSV.Save(FileName);
            }
        }

        private void bRegister_Click(object sender, EventArgs e)
        {
            switch(Mode)
            {
                case ProgramMode.NotConnected:
                    break;
                case ProgramMode.Ready:
                    NL.Send("START\n"); // ok
                    break;
                case ProgramMode.Measure:
                    SaveToCSV();
                    ChangeMode(ProgramMode.Ready);
                    NL.Send("READY\n"); // ok
                    Debug.WriteLine("Saved");
                    break;
            }
        }

        private void ChangeMode(ProgramMode New)
        {
            if (Mode == New) return;

            string Status = "...";
            Mode = New;

            switch (Mode)
            {
                case ProgramMode.NotConnected:
                    Status = "Не подключено";
                    Array.ForEach(Inputs, I => { I.Text = ""; I.Enabled = false; });
                    bRegister.Enabled = false;
                    bTare.Enabled = false;
                    break;
                case ProgramMode.MeasureWait:
                    Status = "Измерение";
                    Array.ForEach(Inputs, I => { I.Text = ""; I.Enabled = false; });
                    bRegister.Enabled = false;
                    bTare.Enabled = false;
                    break;
                case ProgramMode.Measure:
                    Status = "Регистрация";
                    Array.ForEach(Inputs, I => { I.Enabled = true; });
                    bRegister.Enabled = true;
                    bRegister.Focus();
                    bTare.Enabled = false;
                    break;
                case ProgramMode.Ready:
                    Status = "Готовность";
                    Array.ForEach(Inputs, I => { I.Text = ""; I.Enabled = false; });
                    bRegister.Enabled = true;
                    bTare.Enabled = true;
                    break;
                case ProgramMode.Calibrate:
                    Status = "Калибровка";
                    Array.ForEach(Inputs, I => { I.Text = ""; I.Enabled = false; });
                    bRegister.Enabled = false;
                    bTare.Enabled = false;
                    break;
            }

            lStatus.Text = "Статус: " + Status;
        }

        private void bTare_Click(object sender, EventArgs e)
        {
            if(Mode == ProgramMode.Ready)
            {
                ChangeMode(ProgramMode.Calibrate);
                NL.Send("ZEROWEIGHT\n");
            }
        }
    }

    enum ProgramMode
    {
        Unknown,
        NotConnected,
        Ready,
        MeasureWait,
        Measure,
        Calibrate
    }
}
