﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using WSMeter.Link;
using System.Security.Cryptography;

using Extension.Parameter;
using System.IO;

namespace WSMeter
{
    partial class fSettings : Form
    {
        bool Find = false;
        bool Printer = false;
        Link.Link mL;
        Config mC;
        bool DeviceSettings = false;

        public fSettings(Link.Link L, Config C)
        {
            InitializeComponent();

            mL = L;
            mC = C;
        }

        void fSettings_onDeviceFound(object sender, IPEventArgs e)
        {
            if(Find)
            {
                Find = false;
                BeginInvoke((MethodInvoker)delegate
                {
                    tDevIP.Text = e.IP.ToString();
                });
            }
        }

        private void CheckConnection()
        {
            var Ok = mL.IsConnected;

            DeviceSettings = Ok;
            tLength.Enabled = Ok;
            tWidth.Enabled = Ok;
            tVolWeight.Enabled = Ok;
            tHeight.Enabled = Ok;
            tMultiply.Enabled = Ok;
            tTare.Enabled = Ok;
            tKZ.Enabled = Ok;
            tKY.Enabled = Ok;
            tKZ.Enabled = Ok;

            bSwitchService.Enabled = Ok;
            bZeroTable.Enabled = Ok;
        }

        private void fSettings_Load(object sender, EventArgs e)
        {
            mL.StringReceived += L_StringReceived;
            if (mL.GetType() == typeof(NetworkLink))
            {
                (mL as NetworkLink).onDeviceFound += fSettings_onDeviceFound;
            }

            CheckConnection();
            if(mL.IsConnected) mL.Send("ID=SETTINGS\nSETTINGS\n");

            tCSVDir.Text = mC.CSVDir;
            tCSVFile.Text = mC.CSVFile;
            ckSaveToFile.Checked = mC.UseCSVFile;
            tCSVDir.Enabled = !mC.UseCSVFile;
            tCSVFile.Enabled = mC.UseCSVFile;

            tUser1.Text = mC.User1;
            tUser2.Text = mC.User2;
            tUser3.Text = mC.User3;
            cbPorts.Text = mC.SerialPort;
            tDevIP.Text = mC.DeviceIP;
            ckAutosave.Checked = mC.AutoSave;
        }


        private void ckSaveToFile_CheckedChanged(object sender, EventArgs e)
        {
            tCSVDir.Enabled = !ckSaveToFile.Checked;
            tCSVFile.Enabled = ckSaveToFile.Checked;
        }

        private void fSettings_FormClosing(object sender, FormClosingEventArgs e)
        {
            mL.Manual = false;

            mL.StringReceived -= L_StringReceived;
            if (mL.GetType() == typeof(NetworkLink))
            {
                (mL as NetworkLink).onDeviceFound -= fSettings_onDeviceFound;
            }
        }

        void L_StringReceived(object sender, SerialDataEventArgs e)
        {
            BeginInvoke((MethodInvoker)delegate()
            {
                if (mL.Manual)
                    conAddLine(e.Text, false);
                else
                {
                    string ParamName = e.Text.getParameterName();
                    switch (ParamName)
                    {
                        case "OK": break;
                        case "TBLLENGTH": tLength.Text = e.Text.getStringValue("0"); break;
                        case "TBLWIDTH": tWidth.Text = e.Text.getStringValue("0"); break;
                        case "TBLHEIGHT": tHeight.Text = e.Text.getStringValue("0"); break;
                        case "SCLTARE": tTare.Text = e.Text.getStringValue("0"); break;
                        case "SCLMUL": tMultiply.Text = e.Text.getStringValue("0"); break;
                        case "VOLWEIGHT": tVolWeight.Text = e.Text.getStringValue("0"); break;
                        case "KX": tKX.Text = e.Text.getStringValue("0"); break;
                        case "KY": tKY.Text = e.Text.getStringValue("0"); break;
                        case "KZ": tKZ.Text = e.Text.getStringValue("0"); break;
                        case "PRINTER":
                            Printer = true;
                            InitLabel();
                            break;
                        case "VERSION":
                            lVersion.Text = $"Версия устройства: {e.Text.getStringValue("")}";
                            break;
                        case "ID":
                            switch (e.Text.getStringValue(""))
                            {
                                case "PRINTER":
                                    lLabelStatus.ForeColor = Color.Green;
                                    lLabelStatus.Text = "Параметры записаны";
                                    break;
                                case "SETTINGS":
                                    mL.Send("VERSION");
                                    break;
                            }
                            break;
                    }
                }
            });
        }

        private void bSave_Click(object sender, EventArgs e)
        {
            mC.CSVDir = tCSVDir.Text;
            mC.CSVFile = tCSVFile.Text;
            mC.UseCSVFile = ckSaveToFile.Checked;
            mC.User1 = tUser1.Text;
            mC.User2 = tUser2.Text;
            mC.User3 = tUser3.Text;
            mC.SerialPort = cbPorts.Text;
            mC.DeviceIP = tDevIP.Text;
            mC.AutoSave = ckAutosave.Checked;

            // Save
            mC.Save();
        }

        private void bFind_Click(object sender, EventArgs e)
        {
            if(mL.GetType() == typeof(NetworkLink))
            {
                Find = true;
                (mL as NetworkLink).SendBroadcast(cbNetwork.SelectedInterface, "FIND\n"); // ok
            }
        }

        static string GetMd5Hash(MD5 md5Hash, string input)
        {
            byte[] data = md5Hash.ComputeHash(Encoding.UTF8.GetBytes(input));
            StringBuilder sBuilder = new StringBuilder();
            for (int i = 0; i < data.Length; i++)
                sBuilder.Append(data[i].ToString("x2"));
            return sBuilder.ToString();
        }

        private bool IsAllowedService()
        {
            return (bUnlock.Tag != null) && (bUnlock.Tag.ToString().CompareTo(mC.Hash) == 0);
        }

        private void bZeroTable_Click(object sender, EventArgs e)
        {
            if (IsAllowedService())
                mL.Send("ZEROTABLE\n");
        }

        private void bSwitchService_Click(object sender, EventArgs e)
        {
            if (IsAllowedService())
                mL.Send("SERVICE\n");
        }

        private void bUnlock_Click(object sender, EventArgs e)
        {
            using (MD5 md5Hash = MD5.Create())
            {
                string Loc = System.Reflection.Assembly.GetExecutingAssembly().Location;
                string hash = GetMd5Hash(md5Hash, "Measure" + tPassword.Text + "Sensotec");

                bUnlock.Tag = hash;
                tPassword.Text = "";
                
                bool En = (hash.CompareTo(mC.Hash) == 0);
                gbService.Enabled = En;
                gbScaleCal.Enabled = En;
                gbVolume.Enabled = En;
                gbTable.Enabled = En;
                gbUZMul.Enabled = En;
                gbLabelParams.Enabled = En && Printer;
                bSaveCalData.Enabled = En;

                rtConsole.Enabled = En;
                ckManual.Enabled = En;

                if(!En)
                {
                    tSend.Enabled = false;
                    bSend.Enabled = false;
                }
            }
        }

        private void conAddLine(string Text, bool Sended)
        {
            rtConsole.SelectionLength = 0;
            rtConsole.SelectionStart = rtConsole.Text.Length;
            rtConsole.SelectionColor = (Sended) ? Color.DarkBlue : Color.DarkGreen;
            rtConsole.AppendText(Text.Trim() + "\r\n");
        }

        private void tHeight_TextChanged(object sender, EventArgs e)
        {

        }

        private void bSend_Click(object sender, EventArgs e)
        {
            var T = tSend.Text + "\n";
            mL.Send(T);
            rtConsole.ResetText();
            conAddLine(T, true);
            tSend.Text = "";
        }

        private void ckManual_CheckedChanged(object sender, EventArgs e)
        {
            var Ok = ckManual.Enabled && ckManual.Checked && (bUnlock.Tag != null) && (bUnlock.Tag.ToString().CompareTo(mC.Hash) == 0);

            bSend.Enabled = Ok;
            tSend.Enabled = Ok;
            mL.Manual = Ok;
        }

        private void tSend_KeyPress(object sender, KeyPressEventArgs e)
        {
            if((e.KeyChar == 13) && (bSend.Enabled))
                bSend.PerformClick();
        }

        private void bSaveCalData_Click(object sender, EventArgs e)
        {
            string Cmd = "";
            Cmd += $"TBLLENGTH={tLength.Text}\n";
            Cmd += $"TBLWIDTH={tWidth.Text}\n";
            Cmd += $"TBLHEIGHT={tHeight.Text}\n";
            Cmd += $"SCLTARE={tTare.Text}\n";
            Cmd += $"SCLMUL={tMultiply.Text}\n";
            Cmd += $"VOLWEIGHT={tVolWeight.Text}\n";
            Cmd += $"KX={tKX.Text}\n";
            Cmd += $"KY={tKY.Text}\n";
            Cmd += $"KZ={tKZ.Text}\n";
            Cmd += $"SAVE\n";

            if (IsAllowedService())
                mL.Send(Cmd);
        }

        void InitLabel()
        {
            var Files = Directory.GetFiles("./Labels", "*.xml");

            cbLabel.Items.Clear();
            foreach(var F in Files)
            {
                try
                {
                    var Lbl = XMLFile.XMLFile.LoadXML<LabelConfig.LabelCfg>(F);

                    cbLabel.Items.Add(Lbl);
                }
                catch(Exception E)
                {
                    Debug.WriteLine($"Load label file {F} error: {E.Message}");
                }
            }

            if (cbLabel.Items.Count > 0)
                cbLabel.SelectedIndex = 0;
        }

        private void cbLabel_SelectedIndexChanged(object sender, EventArgs e)
        {
            bSendLabel.Enabled = cbLabel.SelectedIndex != -1;
        }

        private void bSendLabel_Click(object sender, EventArgs e)
        {
            var I = cbLabel.SelectedItem;

            lLabelStatus.ForeColor = SystemColors.ControlText;
            lLabelStatus.Text = "...";
            if (I != null)
                mL.Send((I as LabelConfig.LabelCfg).GetCommand());
        }

        private void bRecvLabel_Click(object sender, EventArgs e)
        {
            mL.Send(LabelConfig.LabelCfg.GetQueryCommand());
        }
    }
}
